@@ -31,14 +31,21 @@ sub generate_songbook {
 my $single_space = 0;		# suppress chords line when empty
 my $lyrics_only = 0;		# suppress all chords lines
 my $chords_under = 0;		# chords under lyrics
+my $layout = Text::Layout::Text->new;
+my $rechorus;
+
+sub upd_config {
+    $lyrics_only  = $config->{settings}->{'lyrics-only'};
+    $chords_under = $config->{settings}->{'chords-under'};
+    $rechorus  = $config->{text}->{chorus}->{recall};
+}
 
 sub generate_song {
     my ( $s ) = @_;
 
     my $tidy      = $options->{'backend-option'}->{tidy};
     $single_space = $options->{'single-space'};
-    $lyrics_only  = $config->{settings}->{'lyrics-only'};
-    $chords_under = $config->{settings}->{'chords-under'};
+    upd_config();
 
     $s->structurize
       if ( $options->{'backend-option'}->{structure} // '' ) eq 'structured';
@@ -54,7 +61,9 @@ sub generate_song {
     push(@s, "") if $tidy;
 
     my $ctx = "";
-    foreach my $elt ( @{$s->{body}} ) {
+    my @elts = @{$s->{body}};
+    while ( @elts ) {
+	my $elt = shift(@elts);
 
 	if ( $elt->{context} ne $ctx ) {
 	    push(@s, "-- End of $ctx") if $ctx;
@@ -79,7 +88,7 @@ sub generate_song {
 	}
 
 	if ( $elt->{type} eq "songline" ) {
-	    push(@s, songline($elt));
+	    push(@s, songline( $s, $elt ));
 	    next;
 	}
 
@@ -97,7 +106,7 @@ sub generate_song {
 		    next;
 		}
 		if ( $e->{type} eq "songline" ) {
-		    push(@s, songline($e));
+		    push(@s, songline( $s, $e ));
 		    next;
 		}
 	    }
@@ -106,6 +115,19 @@ sub generate_song {
 	    next;
 	}
 
+	if ( $elt->{type} eq "rechorus" ) {
+	    if ( $rechorus->{quote} ) {
+		unshift( @elts, @{ $elt->{chorus} } );
+	    }
+	    elsif ( $rechorus->{type} &&  $rechorus->{tag} ) {
+		push( @s, "{".$rechorus->{type}.": ".$rechorus->{tag}."}" );
+	    }
+	    else {
+		push( @s, "{chorus}" );
+	    }
+	    next;
+	}
+
 	if ( $elt->{type} eq "tab" ) {
 	    push(@s, "") if $tidy;
 	    push(@s, "-- Start of tab");
@@ -125,7 +147,7 @@ sub generate_song {
 		    next;
 		}
 		if ( $e->{type} eq "songline" ) {
-		    push(@s, songline($e));
+		    push(@s, songline( $s, $e ));
 		    next;
 		}
 		if ( $e->{type} eq "comment" ) {
@@ -177,6 +199,18 @@ sub generate_song {
 		$lyrics_only = $elt->{value}
 		  unless $lyrics_only > 1;
 	    }
+	    # Arbitrary config values.
+	    elsif ( $elt->{name} =~ /^(text\..+)/ ) {
+		my @k = split( /[.]/, $1 );
+		my $cc = {};
+		my $c = \$cc;
+		foreach ( @k ) {
+		    $c = \($$c->{$_});
+		}
+		$$c = $elt->{value};
+		$config->augment($cc);
+		upd_config();
+	    }
 	    next;
 	}
 
@@ -189,21 +223,23 @@ sub generate_song {
 }
 
 sub songline {
-    my ($elt) = @_;
+    my ( $song, $elt ) = @_;
 
     my $t_line = "";
+    my @phrases = map { $layout->set_markup($_); $layout->render }
+      @{ $elt->{phrases} };
 
     if ( $lyrics_only
 	 or
 	 $single_space && ! ( $elt->{chords} && join( "", @{ $elt->{chords} } ) =~ /\S/ )
        ) {
-	$t_line = join( "", @{ $elt->{phrases} } );
+	$t_line = join( "", @phrases );
 	$t_line =~ s/\s+$//;
 	return $t_line;
     }
 
     unless ( $elt->{chords} ) {
-	return ( "", join( " ", @{ $elt->{phrases} } ) );
+	return ( "", join( " ", @phrases ) );
     }
 
     if ( my $f = $::config->{settings}->{'inline-chords'} ) {
@@ -211,16 +247,16 @@ sub songline {
 	$f .= '%s';
 	foreach ( 0..$#{$elt->{chords}} ) {
 	    $t_line .= sprintf( $f,
-				$elt->{chords}->[$_],
-				$elt->{phrases}->[$_] );
+				chord( $song, $elt->{chords}->[$_] ),
+				$phrases[$_] );
 	}
 	return ( $t_line );
     }
 
     my $c_line = "";
     foreach ( 0..$#{$elt->{chords}} ) {
-	$c_line .= $elt->{chords}->[$_] . " ";
-	$t_line .= $elt->{phrases}->[$_];
+	$c_line .= chord( $song, $elt->{chords}->[$_] ) . " ";
+	$t_line .= $phrases[$_];
 	my $d = length($c_line) - length($t_line);
 	$t_line .= "-" x $d if $d > 0;
 	$c_line .= " " x -$d if $d < 0;
@@ -231,4 +267,39 @@ sub songline {
       : ( $c_line, $t_line )
 }
 
+sub chord {
+    my ( $s, $c ) = @_;
+    return "" unless length($c);
+    my $ci = $s->{chordsinfo}->{$c};
+    return "<<$c>>" unless defined $ci;
+    $layout->set_markup($ci->show);
+    my $t = $layout->render;
+    return $ci->is_annotation ? "*$t" : $t;
+}
+
+# Temporary. Eventually we'll have a decent HTML backend for Text::Layout.
+
+package Text::Layout::Text;
+
+use parent 'Text::Layout';
+
+# Eliminate warning when HTML backend is loaded together with Text backend.
+no warnings 'redefine';
+
+sub new {
+    my ( $pkg, @data ) = @_;
+    my $self = $pkg->SUPER::new;
+    $self;
+}
+
+sub render {
+    my ( $self ) = @_;
+    my $res = "";
+    foreach my $fragment ( @{ $self->{_content} } ) {
+	next unless length($fragment->{text});
+	$res .= $fragment->{text};
+    }
+    $res;
+}
+
 1;
